
#pragma comment(lib, "msi.lib")

#include <windows.h>
#include <msi.h>
#include <msiquery.h>

/***************************************************************************
 *
 *	MsiNoUnsupportedAdvertise
 *	  Robert Dickau, InstallShield Software Corp.,
 *	  www.installshield.com/training
 *
 *	Executable to set the "no unsupported advertise" bit for features
 *	  in an MSI database as part of a post-build process.
 *
 ***************************************************************************/

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE, LPSTR szPackagePath, int nShowCmd)
{
	if (lstrlen(szPackagePath) == 0)
	{
		MessageBox(
			NULL,
			TEXT("Drop an MSI database on me, and I'll set the "
				"\"No Unsupported Advertise\" attribute for all its "
				"features."),
			TEXT("Usage"), MB_OK | MB_ICONINFORMATION);

		exit(0);
	}

	if (MsiVerifyPackage(szPackagePath) != ERROR_SUCCESS)
	{
		MessageBox(
			NULL,
			TEXT("You need to specify a valid MSI package."),
			TEXT("Error"),
			MB_OK | MB_ICONERROR);

		exit(1);
	}

	MSIHANDLE hDB = NULL;
	MsiOpenDatabase(szPackagePath, MSIDBOPEN_DIRECT, &hDB);

	MSIHANDLE hView = NULL;
	MsiDatabaseOpenView(hDB, TEXT("SELECT Attributes FROM Feature"), &hView);

	MsiViewExecute(hView, 0);

	MSIHANDLE hRecord = NULL;
	while (MsiViewFetch(hView, &hRecord) == ERROR_SUCCESS)
	{
		INT attr = MsiRecordGetInteger(hRecord, 1);

		// "disallow advertise" (8) and "no unsupported advertise" (32)
		//  are mutually exclusive
		if (!(attr & 8))
		{
			MsiRecordSetInteger(hRecord, 1, attr | 32);
			MsiViewModify(hView, MSIMODIFY_UPDATE, hRecord);
		}
	}

	MsiViewClose(hView);
	MsiDatabaseCommit(hDB);

	MsiCloseHandle(hRecord);
	MsiCloseHandle(hView);
	MsiCloseHandle(hDB);

	return 0;
}